// Copyright (c) 2016 ETH-Zurich. All rights reserved.
// Use of this source code is governed by a BSD-style license that
// can be found in the LICENSE file.

#include "utility/external.h"
#include "communicator.h"

#include "threads.h"
#include "tasks.h"
#include "provider.h"

namespace tinympi {

thread_local Communicator comm_world_;

// at this point, we know that all local tasks have been created. We can now assign them to
// comm_world.
void Communicator::post_init() {

	if (tasks::get_taskunique_id() == 0) {
		// only the root task should set up this
		using namespace impl;
		node_id_t node_id = env::node_id;
		thread_id_t thread_id = env::thread_id;

		//////////////////////
		// CREATE COMM_WORLD
		comm_world_.global_communicator_ = &gcomm_world_;
		comm_world_.task_ranks_ = new rank_type[env::num_tasks];

		// get the area from
		rank_type start_rank = node_id * env::num_threads * env::num_tasks
				+ thread_id * env::num_tasks;
		tasks::Task** local_tasks = &gcomm_world_.task_mapping_[start_rank];

		for (task_id_t k = 0; k < env::num_tasks; ++k) {
			// set up local task -> global rank mapping
			comm_world_.task_ranks_[k] = start_rank + k;

			// initialize gcomm_world task
			local_tasks[k] = &tasks::impl::all_tasks_[thread_id * env::num_tasks + k];
			local_tasks[k]->isend = ThreadWrapper::isend;
			local_tasks[k]->post_knownsource_recv_request = ThreadWrapper::posted_request;
		}
	}
}

void Communicator::finalize() {
	delete[] comm_world_.task_ranks_;
}

}

