// Copyright (c) 2016 ETH-Zurich. All rights reserved.
// Use of this source code is governed by a BSD-style license that
// can be found in the LICENSE file.

#include "global.h"
#include "tasks.h"

#include "global_communicator.h"
#include "provider.h"

namespace tinympi {

GlobalCommunicator gcomm_world_;

////////////////////////////////////
// INITIALIZE GLOBAL COMMUNICATOR
void GlobalCommunicator::pre_init() {
	using namespace impl;

	//////////////////////
	// CREATE GCOMM_WORLD
	gcomm_world_.num_tasks_ = env::num_nodes * env::num_threads
			* env::num_tasks;
	gcomm_world_.task_mapping_ = new tasks::Task*[gcomm_world_.num_tasks_];
	gcomm_world_.id_ = 0;

	// initialize all tasks on foreign nodes
	for (node_id_t i = 0; i < env::num_nodes; ++i) {
		if (i != env::node_id) {
			for (thread_id_t j = 0; j < env::num_threads; ++j) {
				for (task_id_t k = 0; k < env::num_tasks; ++k) {

					// for these we use the node wrapper
					rank_type rank = (env::num_threads * i + j) * env::num_tasks + k;
					gcomm_world_.task_mapping_[rank] = new tasks::Task(i, j, k);
					gcomm_world_.task_mapping_[rank]->isend = NodeWrapper::isend;
					gcomm_world_.task_mapping_[rank]->post_knownsource_recv_request
							= NodeWrapper::posted_request;
				}
			}
		}
	}
}
void GlobalCommunicator::post_finalize() {
	delete[] gcomm_world_.task_mapping_;
}

} // END NAMESPACE tinympi
