// Copyright (c) 2016 ETH-Zurich. All rights reserved.
// Use of this source code is governed by a BSD-style license that
// can be found in the LICENSE file.

#ifndef _EAGER_MESSAGE_H_635106f3226c46d3d114158785752466
#define _EAGER_MESSAGE_H_635106f3226c46d3d114158785752466

#include <atomic>

#include "verbs_config.h"

namespace tinympi {
namespace verbs {
namespace impl {

enum class eager_message_status : uint8_t {
	free,     // slot is free to be uesd
	reserved, // slot is in use
	nackldg,  // slot is used but not acknldg to the sender
	retrieved // slot is filled but not yet reserved by a thread
};

//tinympi::verbs::impl::EagerMessage
struct EagerMessage {
	int8_t buffer[EAGER_MESSAGE_THRESH];                  // x*64 + 38 byte
	context_id_t context_id;                              // 2 byte
	tag_t tag;                                            // 4 byte
	local_id_t dest_id;                                   // 4 byte
	global_id_t src_id;                                   // 8 byte
	rid_t rid;                                            // 4 byte
	eager_size_t buflen;                                  // 2 byte
	piggybackmsg_t piggyback_cnt;                         // 1 byte
	volatile std::atomic<eager_message_status> status;    // 1 byte DO NOT MOVE
};

static_assert(sizeof(EagerMessage) % 64 == 0, "EagerMessage should have a size "
		"which is a multiple of one cache line");

static_assert(std::is_standard_layout<EagerMessage>::value, "EagerMessage must be POD!");

}
}
}

#endif //_EAGER_MESSAGE_H_
