// Copyright (c) 2016 ETH-Zurich. All rights reserved.
// Use of this source code is governed by a BSD-style license that
// can be found in the LICENSE file.

#ifndef _VERBS_MRWRAPPER_H_635106f3226c46d3d114158785752466
#define _VERBS_MRWRAPPER_H_635106f3226c46d3d114158785752466

#include <infiniband/verbs.h>

#include "debug/debugger.h"
#include "utility/error.h"

namespace tinympi {
namespace verbs {
namespace impl {

class mr_wrapper {
public:
	mr_wrapper(ibv_pd* pd, uint64_t start, size_t sz) {
		sz_ = sz;
		mr_ = ibv_reg_mr(pd, reinterpret_cast<void*>(start), sz,
			IBV_ACCESS_REMOTE_WRITE | IBV_ACCESS_LOCAL_WRITE | IBV_ACCESS_REMOTE_READ);
		TMPI_LOG_LEVEL(4, "mr_wrapper", "registered " << mr_ << ": start = " << (void*)start
				<< ", size = " << sz);
		TMPI_ASSERT_FATAL(mr_ != nullptr, "ibv_reg_mr() returned 0", __FILE__, __LINE__);
	}
	~mr_wrapper();
	ibv_mr* get_mr() const { return mr_; }
	size_t get_sz() const { return sz_; }

private:
	size_t sz_;
	ibv_mr* mr_;

};

} // END NAMESPACE impl
} // END NAMESPACE verbs
} // END NAMESPACE tinympi

#endif //_VERBS_MRWRAPPER_H_
