// Copyright (c) 2016 ETH-Zurich. All rights reserved.
// Use of this source code is governed by a BSD-style license that
// can be found in the LICENSE file.

#ifndef _VERBS_RECVREQUEST_H_635106f3226c46d3d114158785752466
#define _VERBS_RECVREQUEST_H_635106f3226c46d3d114158785752466

#include "global.h"
#include "status.h"

#include "verbs_config.h"
#include "verbs_endpoint.h"
#include "verbs_request.h"

namespace tinympi {
namespace verbs {
namespace impl {

#define VERBS_RECV_REQUEST_MEMBERS \
	VERBS_REQUEST_MEMBERS \
	rid_t rid;

//tinympi::verbs::impl::VerbsRecvRequest
struct VerbsRecvRequest {
	VERBS_RECV_REQUEST_MEMBERS
};
static_assert(std::is_standard_layout<VerbsRecvRequest>::value, "VerbsRecvRequest must be POD!");

REQUESTS_CONVERTIBLE(VerbsRecvRequest, VerbsRequest)
REQUESTS_CONVERTIBLE(VerbsRecvRequest, BaseRequest)

ASSERT_VERBS_REQUEST_COMPATIBLE(VerbsRecvRequest)

#define ASSERT_VERBS_RECVREQUEST_COMPATIBLE(TYPE) \
	ASSERT_VERBS_REQUEST_COMPATIBLE(TYPE) \
	VERIFY_COMPATIBILITY(VerbsRecvRequest, TYPE, rid)

namespace verbsRecvRequest {

	inline void init(VerbsRecvRequest* req, verbs_request_types rtype) {
		verbsRequest::init(TO_VerbsRequest(req), rtype);
	}

	inline bool fill_status(VerbsRecvRequest* req, TMPI_Status* status) {
		using tinympi::impl::baseRequest::is_terminated;

		// only fill if this specific request is completed
		if (is_terminated(TO_BaseRequest(req))) {
			status->MPI_SOURCE = comm_world_.get_commrank(req->peer_rank);
			status->MPI_TAG = req->tag;
			status->MPI_ERROR = TMPI_SUCCESS;
			status->size = req->buflen;
			return true;

		} else {
			return false;
		}
	}

} // END NAMESPACE verbsRecvRequest
} // END NAMESPACE impl
} // END NAMESPACE verbs
} // END NAMESPACE tinympi

#endif //_VERBS_RECVREQUEST_H_
