// Copyright (c) 2016 ETH-Zurich. All rights reserved.
// Use of this source code is governed by a BSD-style license that
// can be found in the LICENSE file.

#ifndef _QUEUE_FUNCTIONS_H_635106f3226c46d3d114158785752466
#define _QUEUE_FUNCTIONS_H_635106f3226c46d3d114158785752466

namespace tinympi {
namespace impl {
namespace recvqueues {

namespace dequeue_request {
	struct yes {
	};
	struct no {
	};
}

namespace impl {

	//////////////////////////////////////
	// FIND REQUEST
	// finds a request in a queue
	template<typename Req, typename F, typename dequeue_request_>
	inline Req* find_request(F&&, Req*&, Req*&, Req*&, dequeue_request_);

	// find a request but do not deque
	template<typename Req, typename F>
	inline Req* find_request(F&& match_function, Req*& head, Req*&, Req*& prev, dequeue_request::no) {
		prev = nullptr;
		Req* ptr = head;

		while (ptr != nullptr) {
			if (match_function(ptr)) {
				return ptr;
			}

			prev = ptr;
			ptr = reinterpret_cast<Req*>(ptr->next_request);
		}
		return nullptr;
	}
	template<typename Req, typename F>
	inline Req* find_request(F&& match_function, Req*& head, Req*& tail, Req*& prev,
			dequeue_request::yes) {
		TMPI_LOG_LEVEL(3, "Queues", "find req in queue (head = " << &head << ")");

		if (head == nullptr) {
			TMPI_LOG_LEVEL(2, "recv queues", "recv queue empty");
			return nullptr;
		}

		prev = nullptr;
		Req* ptr = head;
		// first match head
		TMPI_LOG_LEVEL(3, "recv queues", "compare with tag = " << ptr->tag << "; source = "
				<< ptr->peer_rank << " (ANY: " << global_id_t(TMPI_ANY_SOURCE) << "); context = "
				<< ptr->context_id);
		if (match_function(ptr)) {
			head = reinterpret_cast<Req*>(head->next_request);
			return ptr;
		}

		//otherwise go through whole list and store prev
		prev = ptr;
		ptr = reinterpret_cast<Req*>(ptr->next_request);
		while (ptr != nullptr) {
			TMPI_LOG_LEVEL(3, "recv queues", "compare with tag = " << ptr->tag << "; source = "
					<< ptr->peer_rank << "; context = " << ptr->context_id << " req = " << ptr);
			if (match_function(ptr)) {
				// if we have found the request, we can deque it
				TMPI_LOG_LEVEL(2, "recv queues", "found in queue");
				// if this is the tail, then set new tail
				if (ptr->next_request == nullptr)
					tail = prev;

				prev->next_request = ptr->next_request;
				return ptr;
			}

			prev = ptr;
			ptr = reinterpret_cast<Req*>(ptr->next_request);
		}
		// when we didn't find the request, return nullptr
		TMPI_LOG_LEVEL(2, "recv queues", "did not find in queue");
		return nullptr;
	}

} // END NAMESPACE impl

////////////////////////////
// FIND REQUEST
template<typename dequeue_request_, typename Req>
inline Req* find_unexpected_request(const global_id_t& source, tag_t tag,
		context_id_t context_id, Req*& head, Req*& tail, Req*& prev, dequeue_request_) {
	using namespace impl;
	TMPI_LOG_LEVEL(0, "find in unexpected queue", "source = " << source << "; tag = " << tag
			<< "; " << "context_id = " << context_id);
	return find_request(
		[&](Req* ptr) -> bool {
			return ptr->peer_rank == source && ((tag == TMPI_ANY_TAG && ptr->tag >= 0)
					|| ptr->tag == tag) && ptr->context_id == context_id;
		}, head, tail, prev, dequeue_request_());
}

template<typename dequeue_request_, typename Req>
inline Req* find_unexpected_anysource_request(tag_t tag, context_id_t context_id,
		Req*& head, Req*& tail, Req*& prev, dequeue_request_) {
	using namespace impl;
	TMPI_LOG_LEVEL(0, "find in unexpected queue", "source = ANY_SOURCE; tag = " << tag << "; "
			<< "context_id = " << context_id);
	return find_request(
		[&](Req* ptr) -> bool {
			return (ptr->tag >= 0 && (tag == TMPI_ANY_TAG || ptr->tag == tag))
					&& ptr->context_id == context_id;
		}, head, tail, prev, dequeue_request_());
}
template<typename dequeue_request_, typename Req>
inline Req* find_waiting_request(const global_id_t& source, tag_t tag,
	context_id_t context_id, Req*& head, Req*& tail, Req*& prev, dequeue_request_) {
	using namespace impl;
	TMPI_LOG_LEVEL(0, "find in waiting queue", "source = " << source << "; tag = " << tag << "; "
			<< "context_id = " << context_id);
	return find_request(
		[&](Req* ptr) -> bool {
			return (ptr->peer_rank == (global_id_t)TMPI_ANY_SOURCE || ptr->peer_rank == source)
					&& ((ptr->tag == TMPI_ANY_TAG && tag >= 0) || tag == ptr->tag)
					&& context_id == ptr->context_id;
		}, head, tail, prev, dequeue_request_());
}
////////////////////////////
// ADD REQUEST
template<typename Req>
inline void add_to_queue(Req* req, Req*& head, Req*& tail) {
	TMPI_LOG_LEVEL(3, "Queues", "add req " << req << " to queue (head = " << &head << ")");
	if (head == nullptr) {
		// empty queue
		head = req;
		tail = req;
	} else {
		tail->next_request = reinterpret_cast<BaseRequest*>(req);
		tail = req;
	}
	req->next_request = nullptr;
}
////////////////////////////
// REMOVE REQUEST
// this functions only work locally!
template<typename Req>
inline bool remove_waiting_request(Req* req, Req*& head, Req*& tail) {
	using namespace impl;
	TMPI_LOG_LEVEL(3, "Queues", "remove req " << req << " from queue (head = " << &head << ")");
	Req* prev;
	bool b = find_request([&](Req* ptr) { return ptr == req; }, head, tail, prev,
			dequeue_request::yes());
	TMPI_LOG_LEVEL(3, "remove_waiting_request", (b ? "successfully removed " : "did not remove ")
			<< req);
	TMPI_LOG_LEVEL(3, "remove_waiting_request", "new head: " << head);
	return b;
}
template<typename Req>
inline bool remove_unexpected_request(Req* req, Req*& head, Req*& tail) {
	TMPI_LOG_LEVEL(3, "Queues", "remove req " << req << " from queue (head = " << &head << ")");
	using namespace impl;
	Req* prev;
	bool b = find_request([&](Req* ptr) { return ptr == req; }, head, tail, prev,
			dequeue_request::yes());
	TMPI_LOG_LEVEL(3, "remove_unexpected_request", (b ? "successfully removed " : "did not remove ")
			<< req);
	TMPI_LOG_LEVEL(3, "remove_unexpected_request", "new head: " << head);
	return b;
}


} // END NAMESPACE recvqueues
} // END NAMESPACE impl
} // END NAMESPACE tinympi

#endif // _QUEUE_FUNCTIONS_H_
