#define SAVE_REGISTERS_TO_STACK \
	push %rdi; \
	push %rbp; \
	push %rbx; \
	push %r12; \
	push %r13; \
	push %r14; \
	push %r15; \
	sub $8, %rsp; \
	stmxcsr 4(%rsp); \
	fnstcw (%rsp);

#define RESTORE_REGISTERS_FROM_STACK \
	fldcw (%rsp); \
	ldmxcsr 4(%rsp); \
	add $8, %rsp; \
	pop %r15; \
	pop %r14; \
	pop %r13; \
	pop %r12; \
	pop %rbx; \
	pop %rbp; \
	pop %rdi;

.globl _copythreadstack
_copythreadstack:
	// just for this function we would not need to store the registers, but if we restore the stack
	// on another core, we will need to restore all necessary registers
	SAVE_REGISTERS_TO_STACK

	// INITIALIZED:
	// %rdi := state array
	// %rsi := stack_top

	// %rsi := size of area
	mov %rsi, %r14
	sub %rsp, %rsi

	// state_array[0] := size of area
	mov %rsi, (%rdi)
	add $8, %rdi

	// state_array[1] := rsp
	mov %rsp, (%rdi)
	add $8, %rdi

	// %rax := counter
	mov %rsp, %rax
copyloop:
	// state_array[i+2] := stack[i]
	mov (%rax), %r12
	mov %r12, (%rdi)
	// i += 1
	add $8, %rax
	add $8, %rdi

	// repeat as long below top
	cmp %rax, %r14
	jg copyloop

	RESTORE_REGISTERS_FROM_STACK

	// set our return value. This is necessary because we will return "at this place" also from
	// restorestate, but there we will return a non-nullptr
	mov $0, %rax

	ret $0

.globl _restorestack
_restorestack:
	// rdi := the stackptr we want to swap in
	// rsi := the address we want to store the current stack
	// rdx := one value which survives in the new stack, which allows to distinguish from the other
	//        threads
	// the same as _swapstacks for the tasks (except that we use %rdi and not (%rdi)))
	SAVE_REGISTERS_TO_STACK

	mov %rsp, (%rsi)
	mov %rdi, %rsp

	RESTORE_REGISTERS_FROM_STACK

	// set our return value
	mov %rdx, %rax
	// note that since we are in the old stack we will return to the instruction
	// we have been earlier
	ret $0

.globl _recoverthreadstack
_recoverthreadstack:
	// we are not interested in the old stack which gets deleted anyway,
	// so just swap in the old stack and restore the registers
	mov %rdi, %rsp
	RESTORE_REGISTERS_FROM_STACK
	ret $0

.globl _debugwait
_debugwait:
	nop
