// Copyright (c) 2016 ETH-Zurich. All rights reserved.
// Use of this source code is governed by a BSD-style license that
// can be found in the LICENSE file.

#ifndef _ERROR_H_635106f3226c46d3d114158785752466
#define _ERROR_H_635106f3226c46d3d114158785752466
#include "debug/debugger.h"

#define TMPI_FAIL(description) \
	{ \
		std::stringstream ss; \
		ss << std::left << FAILED_STRING << description << "\n"; \
		Debugger::log(ss.str()); \
	}

#define ERROR_H_COLOR_REDBOLD "\x1b[31;1m"
#define ERROR_H_COLOR_REDNORMAL "\x1b[0m\x1b[31m"

namespace tinympi {
	namespace impl {
		static int fatal_error(std::string text, std::string file, int line) {
			std::stringstream ss;
			ss << ERROR_H_COLOR_REDBOLD << "FATAL ERROR: " << ERROR_H_COLOR_REDNORMAL << text
					<< " in file " << file << " (Line " << line << ")\n";
			Debugger::log(ss.str());
			exit(1);
		}
	}
}
#define TMPI_FATAL_ERROR_EFF(text, file, line) \
	tinympi::impl::fatal_error(text, file, line)
#define TMPI_ASSERT_FATAL_EFF(cond, text, file, line) \
	if (__builtin_expect(!(cond),0)) { \
		tinympi::impl::fatal_error(text, file, line); \
	}

#define TMPI_FATAL_ERROR(text, file, line) \
	{ \
	std::stringstream ss; \
	ss << ERROR_H_COLOR_REDBOLD << "FATAL ERROR: " << ERROR_H_COLOR_REDNORMAL << text \
			<< " in file " << file << " (Line " << line << ")\n"; \
	Debugger::log(ss.str()); \
	exit(1); \
	}

#define TMPI_ASSERT_FATAL(cond, text, file, line) \
	if (!(cond)) { TMPI_FATAL_ERROR(text, file, line); }
#ifdef NO_ASSERT_FATAL
	#define TMPI_DASSERT_FATAL(a,b,c,d) {}
#else
	#define TMPI_DASSERT_FATAL(a,b,c,d) TMPI_ASSERT_FATAL(a,b,c,d)
#endif

#endif //_ERROR_H_
