// Copyright (c) 2016 ETH-Zurich. All rights reserved.
// Use of this source code is governed by a BSD-style license that
// can be found in the LICENSE file.

#ifndef _EXTERNAL_H_635106f3226c46d3d114158785752466
#define _EXTERNAL_H_635106f3226c46d3d114158785752466

#include <cassert>
#include <fstream>

#include "mpi.h"

#include "global.h"
#include "utility/error.h"

namespace tinympi {
namespace tasks {

bool is_master();

} // END NAMESPACE tasks

namespace utility {
namespace external {


inline int allgather(void* sendbuf, int  sendcount, MPI_Datatype sendtype, void* recvbuf,
		int recvcount, MPI_Datatype recvtype, MPI_Comm comm) {

	return MPI_Allgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);

}

inline int all2all(void* sendbuf, int sendcount, MPI_Datatype sendtype,
		void* recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm) {

	return MPI_Alltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);

}

struct ProcessInfo {
	int numprocs;
	int pid;

};
extern ProcessInfo pi_;

inline void pre_init(int* argc, char*** argv) {
	int provided;
	MPI_Init_thread(argc, argv, MPI_THREAD_FUNNELED, &provided);

	MPI_Comm_size(MPI_COMM_WORLD, &pi_.numprocs);
	MPI_Comm_rank(MPI_COMM_WORLD, &pi_.pid);
}
inline void init() {
	lsb_init();
}
inline void finalize() {
	lsb_finalize();
}
inline void post_finalize() {

	MPI_Barrier(MPI_COMM_WORLD);
	int res = MPI_Finalize();
	TMPI_ASSERT_FATAL_EFF(res == 0, "MPI_Finalize() failed", __FILE__, __LINE__);
}

inline unsigned get_numnodes() {
	return pi_.numprocs;
}
inline node_id_t get_noderank() {
	return pi_.pid;
}


}
}
}

#endif //_EXTERNAL_H_
